#!/usr/bin/env python

from distutils.core import setup

setup(
    name='jsonrpctcp',
    version='0.3',
    description='JSON-RPC (v2) over TCP sockets.',
    author='Josh Marshall',
    author_email='catchjosh@gmail.com',
    packages=['jsonrpctcp',]
)

